"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@collectable/core");
const red_black_tree_1 = require("@collectable/red-black-tree");
const map_1 = require("@collectable/map");
const values_1 = require("./values");
const functions_1 = require("../functions");
const iterate_1 = require("./iterate");
class SortedMapStructure {
    /** @internal */
    constructor(mctx, _indexed, _sorted, _compare, _select) {
        this._indexed = _indexed;
        this._sorted = _sorted;
        this._compare = _compare;
        this._select = _select;
        this['@@mctx'] = mctx;
    }
    /** @internal */
    get '@@size'() { return this._indexed['@@size']; }
    /** @internal */
    get '@@is-collection'() { return true; }
    /** @internal */
    '@@clone'(mctx) {
        var sctx = core_1.Mutation.asSubordinateContext(mctx);
        return new SortedMapStructure(mctx, core_1.Mutation.clone(this._indexed, sctx), core_1.Mutation.clone(this._sorted, sctx), this._compare, this._select);
    }
    /** @internal */
    '@@equals'(other) {
        return functions_1.isEqual(this, other);
    }
    /** @internal */
    '@@hash'() {
        return core_1.hashIterator(iterate_1.iteratePairs(this));
    }
    /** @internal */
    '@@unwrap'() {
        return core_1.unwrap(this);
    }
    /** @internal */
    '@@unwrapInto'(target) {
        var it = red_black_tree_1.RedBlackTree.iterateFromFirst(this._sorted);
        var current;
        while (!(current = it.next()).done) {
            var entry = current.value;
            target[core_1.unwrapKey(entry.key.key)] = core_1.unwrap(entry.value);
        }
        return target;
    }
    /** @internal */
    '@@createUnwrapTarget'() {
        return {};
    }
    /** @internal */
    '@@get'(key) {
        return functions_1.get(key, this);
    }
    /** @internal */
    '@@has'(key) {
        return functions_1.has(key, this);
    }
    /** @internal */
    '@@set'(key, value) {
        return functions_1.set(key, value, this);
    }
    /** @internal */
    '@@update'(updater, key) {
        return functions_1.update(updater, key, this);
    }
    /** @internal */
    '@@verifyKey'(key) {
        return true;
    }
    [Symbol.iterator]() {
        return iterate_1.iteratePairs(this);
    }
}
exports.SortedMapStructure = SortedMapStructure;
function isSortedMap(arg) {
    return core_1.isObject(arg) && arg instanceof SortedMapStructure;
}
exports.isSortedMap = isSortedMap;
function cloneSortedMap(map, clear, mutability) {
    var mctx = core_1.Mutation.selectContext(mutability);
    var sctx = core_1.Mutation.asSubordinateContext(mctx);
    var keys;
    var sorted;
    if (clear) {
        keys = map_1.HashMap.empty(sctx);
        sorted = red_black_tree_1.RedBlackTree.empty(map._compare, sctx);
    }
    else {
        keys = core_1.Mutation.clone(map._indexed, sctx);
        sorted = core_1.Mutation.clone(map._sorted, sctx);
    }
    return new SortedMapStructure(mctx, keys, sorted, map._compare, map._select);
}
exports.cloneSortedMap = cloneSortedMap;
function createMap(values, compare, select, mutability) {
    var map = core_1.Mutation.modify(emptySortedMap(compare, select, mutability));
    var indexed = map._indexed;
    var sorted = map._sorted;
    if (Array.isArray(values)) {
        for (var i = 0; i < values.length; i++) {
            var value = values[i];
            values_1.setItem(value[0], value[1], indexed, sorted, select);
        }
    }
    else {
        var it = values[Symbol.iterator]();
        var current;
        while (!(current = it.next()).done) {
            value = current.value;
            values_1.setItem(value[0], value[1], indexed, sorted, select);
        }
    }
    return core_1.Mutation.commit(map);
}
exports.createMap = createMap;
function extractTree(set) {
    return set._sorted;
}
exports.extractTree = extractTree;
function extractMap(set) {
    return set._indexed;
}
exports.extractMap = extractMap;
const DEFAULT_COMPARATOR = (a, b) => a.index - b.index;
const COMPARATOR_CACHE = new WeakMap();
function createComparatorFn(compare) {
    var fn = COMPARATOR_CACHE.get(compare);
    return core_1.isDefined(fn) ? fn : (fn = function (a, b) {
        var n = compare(a, b);
        return n === 0 ? DEFAULT_COMPARATOR(a, b) : n;
    }, COMPARATOR_CACHE.set(compare, fn), fn);
}
function emptySortedMap(compare, select, mutability) {
    var comparator = core_1.isDefined(compare) ? createComparatorFn(compare) : DEFAULT_COMPARATOR;
    var mctx = core_1.Mutation.selectContext(mutability);
    if (core_1.Mutation.isImmutableContext(mctx) && comparator === DEFAULT_COMPARATOR) {
        return core_1.isDefined(EMPTY) ? EMPTY : (EMPTY = new SortedMapStructure(mctx, map_1.HashMap.empty(), red_black_tree_1.RedBlackTree.empty(comparator), comparator, void 0));
    }
    var sctx = core_1.Mutation.asSubordinateContext(mctx);
    var map = map_1.HashMap.empty(sctx);
    var tree = red_black_tree_1.RedBlackTree.empty(comparator, sctx);
    return new SortedMapStructure(mctx, map, tree, comparator, select);
}
exports.emptySortedMap = emptySortedMap;
var EMPTY;

//# sourceMappingURL=SortedMap.js.map
