"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@collectable/core");
const map_1 = require("@collectable/map");
const red_black_tree_1 = require("@collectable/red-black-tree");
var _nextIndex = 0;
function setItem(key, value, keyMap, sortedValues, select) {
    var entry;
    map_1.HashMap.update(arg => {
        if (core_1.isUndefined(arg)) {
            return entry = {
                index: ++_nextIndex,
                view: core_1.isDefined(select) ? select(value, key) : void 0,
                key,
                value
            };
        }
        return core_1.isEqual(value, arg.value) ? arg : entry = {
            index: arg.index,
            view: core_1.isDefined(select) ? select(value, key) : void 0,
            key,
            value
        };
    }, key, keyMap);
    if (core_1.isDefined(entry)) {
        red_black_tree_1.RedBlackTree.set({
            index: entry.index,
            view: entry.view,
            key
        }, value, sortedValues);
        return true;
    }
    return false;
}
exports.setItem = setItem;
function unsetItem(key, keyMap, sortedValues) {
    var entry;
    map_1.HashMap.update(arg => {
        if (core_1.isDefined(arg))
            entry = arg;
        return void 0;
    }, key, keyMap);
    if (core_1.isDefined(entry)) {
        red_black_tree_1.RedBlackTree.remove(entry, sortedValues);
        return true;
    }
    return false;
}
exports.unsetItem = unsetItem;
function getItemByKey(key, map) {
    return map_1.HashMap.get(key, map);
}
exports.getItemByKey = getItemByKey;
function getItemByIndex(index, sorted) {
    const size = red_black_tree_1.size(sorted);
    return fromSortedEntry(red_black_tree_1.at(normalizeIndex(size, index), sorted));
}
exports.getItemByIndex = getItemByIndex;
function getFirstItem(sorted) {
    return fromSortedEntry(red_black_tree_1.first(sorted));
}
exports.getFirstItem = getFirstItem;
function getLastItem(sorted) {
    return fromSortedEntry(red_black_tree_1.last(sorted));
}
exports.getLastItem = getLastItem;
function fromSortedEntry(entry) {
    return core_1.isUndefined(entry) ? void 0 : [entry.key.key, entry.value];
}
function normalizeIndex(size, index) {
    return core_1.max(-1, core_1.min(size, index < 0 ? size + index : index));
}

//# sourceMappingURL=values.js.map
