import { isDefined, isUndefined, isEqual, min, max } from '@collectable/core';
import { HashMap } from '@collectable/map';
import { RedBlackTree, size as rbtSize, at, first, last } from '@collectable/red-black-tree';
var _nextIndex = 0;
export function setItem(key, value, keyMap, sortedValues, select) {
    var entry;
    HashMap.update(arg => {
        if (isUndefined(arg)) {
            return entry = {
                index: ++_nextIndex,
                view: isDefined(select) ? select(value, key) : void 0,
                key,
                value
            };
        }
        return isEqual(value, arg.value) ? arg : entry = {
            index: arg.index,
            view: isDefined(select) ? select(value, key) : void 0,
            key,
            value
        };
    }, key, keyMap);
    if (isDefined(entry)) {
        RedBlackTree.set({
            index: entry.index,
            view: entry.view,
            key
        }, value, sortedValues);
        return true;
    }
    return false;
}
export function unsetItem(key, keyMap, sortedValues) {
    var entry;
    HashMap.update(arg => {
        if (isDefined(arg))
            entry = arg;
        return void 0;
    }, key, keyMap);
    if (isDefined(entry)) {
        RedBlackTree.remove(entry, sortedValues);
        return true;
    }
    return false;
}
export function getItemByKey(key, map) {
    return HashMap.get(key, map);
}
export function getItemByIndex(index, sorted) {
    const size = rbtSize(sorted);
    return fromSortedEntry(at(normalizeIndex(size, index), sorted));
}
export function getFirstItem(sorted) {
    return fromSortedEntry(first(sorted));
}
export function getLastItem(sorted) {
    return fromSortedEntry(last(sorted));
}
function fromSortedEntry(entry) {
    return isUndefined(entry) ? void 0 : [entry.key.key, entry.value];
}
function normalizeIndex(size, index) {
    return max(-1, min(size, index < 0 ? size + index : index));
}

//# sourceMappingURL=values.js.map
