import { Mutation, Associative, IndexedCollection, ComparatorFn, KeyedSelectorFn } from '@collectable/core';
import { RedBlackTree } from '@collectable/red-black-tree';
import { HashMap } from '@collectable/map';
import { Entry, SortingKey } from './types';
export declare class SortedMapStructure<K, V, U = any> implements IndexedCollection<K, V, [K, V], Associative<V>> {
    _indexed: HashMap.Instance<K, Entry<K, V, U>>;
    _sorted: RedBlackTree.Instance<SortingKey<K, U>, V>;
    _compare: ComparatorFn<Entry<K, V, U>>;
    _select: KeyedSelectorFn<K, V, U> | undefined;
    /** @internal */
    constructor(mctx: Mutation.Context, _indexed: HashMap.Instance<K, Entry<K, V, U>>, _sorted: RedBlackTree.Instance<SortingKey<K, U>, V>, _compare: ComparatorFn<Entry<K, V, U>>, _select: KeyedSelectorFn<K, V, U> | undefined);
    /** @internal */
    readonly '@@mctx': Mutation.Context;
    /** @internal */
    readonly '@@size': number;
    /** @internal */
    readonly '@@is-collection': true;
    /** @internal */
    '@@clone'(mctx: Mutation.Context): SortedMapStructure<K, V, U>;
    /** @internal */
    '@@equals'(other: SortedMapStructure<K, V, U>): boolean;
    /** @internal */
    '@@hash'(): number;
    /** @internal */
    '@@unwrap'(): Associative<V>;
    /** @internal */
    '@@unwrapInto'(target: Associative<V>): Associative<V>;
    /** @internal */
    '@@createUnwrapTarget'(): Associative<V>;
    /** @internal */
    '@@get'(key: K): V | undefined;
    /** @internal */
    '@@has'(key: K): boolean;
    /** @internal */
    '@@set'(key: K, value: V): this;
    /** @internal */
    '@@update'(updater: (value: V, map: this) => any, key: K): this;
    /** @internal */
    '@@verifyKey'(key: K): boolean;
    [Symbol.iterator](): IterableIterator<[K, V]>;
}
export declare function isSortedMap<K, V, U>(arg: any): arg is SortedMapStructure<K, V, U>;
export declare function cloneSortedMap<K, V, U>(map: SortedMapStructure<K, V, U>, clear?: boolean, mutability?: Mutation.PreferredContext): SortedMapStructure<K, V, U>;
export declare function createMap<K, V, U>(values: [K, V][] | Iterable<[K, V]>, compare?: ComparatorFn<Entry<K, V, U>>, select?: KeyedSelectorFn<K, V, U>, mutability?: Mutation.PreferredContext): SortedMapStructure<K, V, U>;
export declare function extractTree<K, V, U>(set: SortedMapStructure<K, V, U>): RedBlackTree.Instance<SortingKey<K, U>, V>;
export declare function extractMap<K, V, U>(set: SortedMapStructure<K, V, U>): HashMap.Instance<K, Entry<K, V, U>>;
export declare function emptySortedMap<K, V, U = any>(compare?: ComparatorFn<Entry<K, V, U>>, select?: KeyedSelectorFn<K, V, U>, mutability?: Mutation.PreferredContext): SortedMapStructure<K, V, U>;
